import os
import shutil
import click
import pkg_resources

# Define la estructura de directorios base
BASE_STRUCTURE = {
    "app": [
        "crud/schema",
        "database/connection",
        "database/listener",
        "external_api/connection",
        "external_api/firebase",
        "external_api/gmail",
        "lib/sqlacodegen_v2",
        "model",
        "module/smtp",
        "module/telegram",
        "module/whatsapp",
        "resource/public/css",
        "resource/public/ico",
        "resource/public/icons",
        "resource/public/img",
        "resource/public/js",
        "resource/template/email",
        "resource/view",
        "service/middleware/audit",
        "service/middleware/auth",
        "service/middleware/notification/vector",
        "service/request/funtionality",
        "service/request/sys",
        "service/request/web",
        "service/response",
        "util",
    ],
    "script": [
        "deploy",
        "generator/schemas",
        "migrate",
    ],
    "tests": [
        "crud",
        "database",
        "external_api",
        "util",
    ],
}


def create_directories(base_path, structure):
    """Crea la estructura de directorios base"""
    for folder, subfolders in structure.items():
        folder_path = os.path.join(base_path, folder)
        os.makedirs(folder_path, exist_ok=True)
        for subfolder in subfolders:
            subfolder_path = os.path.join(folder_path, subfolder)
            os.makedirs(subfolder_path, exist_ok=True)


def copy_files(src, dst):
    """Copia los archivos de una estructura base al destino"""
    for foldername, _, filenames in os.walk(src):
        for filename in filenames:
            src_file = os.path.join(foldername, filename)
            # Construir la ruta correspondiente en el nuevo proyecto
            relative_path = os.path.relpath(src_file, src)
            dst_file = os.path.join(dst, relative_path)
            dst_folder = os.path.dirname(dst_file)

            # Asegurarse de que el directorio de destino exista
            os.makedirs(dst_folder, exist_ok=True)
            shutil.copy2(src_file, dst_file)


@click.command()
@click.argument("project_name")
def main(project_name):
    """Genera una nueva estructura de proyecto con archivos base"""
    if os.path.exists(project_name):
        click.echo(f"El directorio '{project_name}' ya existe.")
        return

    os.makedirs(project_name)

    # Crear directorios base
    create_directories(project_name, BASE_STRUCTURE)

    # Copiar archivos base desde la carpeta de plantillas
    for folder in BASE_STRUCTURE.keys():
        for subfolder in BASE_STRUCTURE[folder]:
            subfolder_path = os.path.join(project_name, folder, subfolder)
            template_folder = pkg_resources.resource_filename(
                "genapp.template", f"{folder}/{subfolder}"
            )
            if os.path.exists(template_folder):
                copy_files(template_folder, subfolder_path)

    click.echo(f"Estructura del proyecto '{project_name}' creada exitosamente.")


if __name__ == "__main__":
    main()
